/** @format */

import path from 'path';
import fs from 'fs-extra';
import _ from 'lodash';
import { spawn } from 'child_process';
// import yaml from 'js-yaml';
import { loadApplication } from '@serverless-devs/core';
import colors from 'chalk';
import { logger, getYamlPath, common } from '../utils';
import { DEFAULT_REGIRSTRY } from '../constants/static-variable';
const { replaceTemplate, replaceFun, getTemplatekey } = common;

export class InitManager {
  protected promps: any = {};
  constructor() { }

  async initSconfig(appSath) {
    const sPath = getYamlPath(appSath, 's');
    if (sPath) {
      let sContent = fs.readFileSync(sPath, 'utf-8');
      sContent = replaceFun(sContent, { access: 'default' });
      fs.writeFileSync(sPath, sContent, 'utf-8');
    }
    return sPath;
  }

  async assemblySpecialApp(appName, { projectName, appPath }) {
    let result = [];
    if (appName === 'start-component' || appName === 'devsapp/start-component') {
      const packageJsonPath = path.join(appPath, 'package.json');
      const publishYamlPath = path.join(appPath, 'publish.yaml');
      replaceTemplate([packageJsonPath, publishYamlPath], { projectName });
    } else {
      let sYaml = path.join(appPath, 's.yaml');
      if (!fs.existsSync(sYaml)) {
        sYaml = path.join(appPath, 's.yml');
      }
      result = getTemplatekey(fs.readFileSync(sYaml, 'utf-8'));
    }
    return result;

  }

  async executeInit(name: string, dir?: string, projectName?: string): Promise<[]> {
    let result: any = [];
    const registry = DEFAULT_REGIRSTRY;
    process.env.skipPrompt = 'Y'; // 跳过提示
    let appPath = await loadApplication({ registry, target: dir, source: name, name: projectName });
    if (appPath) {
      await this.initSconfig(appPath);
      result = await this.assemblySpecialApp(name, { projectName: path.basename(dir), appPath }); // Set some app template content
      logger.success('\n🏄‍ Thanks for using Serverless-Devs');
      if(result.length >0) {
        console.log(`👉 It is detected that you have additional configuration to be set, please click "Configuration Parameters" to configure`);
      }else {
        console.log(`👉 Congratulations on completing the project initialization, click "Enter Configuration" to continue the next step`);
      } 
      console.log('💞 Document ❤ Star：' + colors.cyan('https://github.com/Serverless-Devs/Serverless-Devs' + '\n'));
    }
    return result;
  }
  async gitCloneProject(name: string, dir?: string) {
    return new Promise(resolve => {
      const gitCmd = spawn('git', ['clone', name], {
        shell: true,
        cwd: dir ? dir : './',
        stdio: ['ignore', 'inherit', 'inherit'],
      });
      gitCmd.on('close', code => {
        resolve({ code });
      });
    });
  }

  async init(name: string, dir?: string, project?: string): Promise<[]> {
    return await this.executeInit(name, dir, project);
  }
}
